package org.eclipse.cdt.managedbuilder.ui.properties;
/*******************************************************************************
 * Copyright (c) 2004 IBM Rational Software Corporation and others. All rights
 * reserved. This program and the accompanying materials are made available
 * under the terms of the Common Public License v0.5 which accompanies this
 * distribution, and is available at http://www.eclipse.org/legal/cpl-v05.html
 * 
 * Contributors: IBM Rational Software - Initial API and implementation
 ******************************************************************************/
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.eclipse.cdt.managedbuilder.core.BuildException;
import org.eclipse.cdt.managedbuilder.core.IConfiguration;
import org.eclipse.cdt.managedbuilder.core.IOption;
import org.eclipse.cdt.managedbuilder.core.ITool;
import org.eclipse.cdt.managedbuilder.core.ManagedBuildManager;
import org.eclipse.cdt.managedbuilder.internal.core.ToolReference;
import org.eclipse.cdt.managedbuilder.internal.ui.ManagedBuilderUIPlugin;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.graphics.Point;

public class BuildToolSettingsPage extends BuildSettingsPage {
	// Field editor label
	private static final String COMMAND = "FieldEditors.tool.command"; //$NON-NLS-1$
	// option names that stores additional options
	private static final String COMPILER_FLAGS = "Other flags";
	private static final String LINKER_FLAGS = "Linker flags";
	// field editor that displays all the build options for a particular tool
	private MultiLineTextFieldEditor allOptionFieldEditor;
	private static final String DEFAULT_SEPERATOR = ";";
	// Tool the settings belong to
	private ITool tool;
	// all build options preference store id
	private String allOptionsId = "";
	BuildToolSettingsPage(IConfiguration configuration, ITool tool) {
		// Cache the configuration and tool this page is for
		super(configuration);
		this.tool = tool;
		allOptionsId = tool.getId() + ".allOptions";
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#computeSize()
	 */
	public Point computeSize() {
		return super.computeSize();
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		// Load up the preference store
		super.createFieldEditors();
		// Add a string editor to edit the tool command
		StringFieldEditor stringField = new StringFieldEditor(tool.getId(),
				ManagedBuilderUIPlugin.getResourceString(COMMAND),
				getFieldEditorParent());
		stringField.setEmptyStringAllowed(false);
		addField(stringField);
		// Add a field editor that displays over all build options
		allOptionFieldEditor = new MultiLineTextFieldEditor(allOptionsId,
				"All options", getFieldEditorParent());
		getPreferenceStore().setValue(allOptionsId, "");
		addField(allOptionFieldEditor);
	}
	/**
	 * Update the field editor that displays all the build options
	 */
	public void updateAllOptionField() {
		try {
			if (getToolFlags() != null) {
				getPreferenceStore().setValue(allOptionsId, getToolFlags());
				allOptionFieldEditor.load();
			}
		} catch (BuildException e) {
		}
	}
	/**
	 * Returns all the build options string
	 * 
	 * @return @throws
	 *         BuildException
	 */
	private String getToolFlags() throws BuildException {
		StringBuffer buf = new StringBuffer();
		// get the options for this tool
		IOption[] options = tool.getOptions();
		String listStr = "";
		String[] listVal = null;
		for (int k = 0; k < options.length; k++) {
			IOption option = options[k];
			switch (option.getValueType()) {
				case IOption.BOOLEAN :
					if (getPreferenceStore().getBoolean(option.getId())) {
						buf.append(option.getCommand() + ITool.WHITE_SPACE);
					}
					break;
				case IOption.ENUMERATED :
					String enumCommand = getPreferenceStore().getString(
							option.getId());
					if (enumCommand.indexOf(DEFAULT_SEPERATOR) != -1)
						enumCommand = option.getSelectedEnum();
					String enum = option.getEnumCommand(enumCommand);
					if (enum.length() > 0) {
						buf.append(enum + ITool.WHITE_SPACE);
					}
					break;
				case IOption.STRING :
					String val = getPreferenceStore().getString(option.getId());
					if (val.length() > 0) {
						buf.append(val + ITool.WHITE_SPACE);
					}
					break;
				case IOption.STRING_LIST :
				case IOption.INCLUDE_PATH :
				case IOption.PREPROCESSOR_SYMBOLS :
				case IOption.LIBRARIES :
				case IOption.OBJECTS :
					String cmd = option.getCommand();
					listStr = getPreferenceStore().getString(option.getId());
					listVal = BuildToolsSettingsStore.parseString(listStr);
					for (int j = 0; j < listVal.length; j++) {
						String temp = listVal[j];
						if (cmd != null)
							buf.append(cmd + temp + ITool.WHITE_SPACE);
						else
							buf.append(temp + ITool.WHITE_SPACE);
					}
					break;
				default :
					break;
			}
		}
		return buf.toString().trim();
	}
	/**
	 * Creates single string from the string array with a separator
	 * 
	 * @param items
	 * @return
	 */
	private String createList(String[] items) {
		StringBuffer path = new StringBuffer(""); //$NON-NLS-1$
		for (int i = 0; i < items.length; i++) {
			path.append(items[i]);
			if (i < (items.length - 1)) {
				path.append(DEFAULT_SEPERATOR);
			}
		}
		return path.toString();
	}
	/**
	 * This method parses the string that is entered in the all build option
	 * field editor and stores the options to the corresponding option fields.
	 */
	public void parseAllOptions() {
		// Get the all build options string from all options field
		String alloptions = getPreferenceStore().getString(allOptionsId);
		// list that holds the options for the option type other than
		// boolean,string and enumerated
		List optionsList = new ArrayList();
		// additional options buffer
		StringBuffer addnOptions = new StringBuffer();
		// split all build options string
		String[] optionsArr = alloptions.split(" ");
		for (int j = 0; j < optionsArr.length; j++) {
			boolean optionValueExist = false;
			// get the options for this tool
			IOption[] options = tool.getOptions();
			for (int k = 0; k < options.length; ++k) {
				IOption opt = options[k];
				String name = opt.getId();
				// check whether the option value is already exist
				// and also change the preference store based on
				// the option value
				switch (opt.getValueType()) {
					case IOption.BOOLEAN :
						if (opt.getCommand().equals(optionsArr[j])) {
							getPreferenceStore().setValue(opt.getId(), true);
							optionValueExist = true;
						}
						break;
					case IOption.ENUMERATED :
						String enum = "";
						String[] enumValues = opt.getApplicableValues();
						for (int i = 0; i < enumValues.length; i++) {
							if (opt.getEnumCommand(enumValues[i]).equals(
									optionsArr[j])) {
								enum = enumValues[i];
								optionValueExist = true;
							}
						}
						if (!enum.equals(""))
							getPreferenceStore().setValue(opt.getId(), enum);
						break;
					case IOption.STRING :
						String str = getPreferenceStore()
								.getString(opt.getId());
						if (alloptions.indexOf(str) == -1) {
							getPreferenceStore().setValue(opt.getId(), "");
						}
						if (str.indexOf(optionsArr[j]) != -1) {
							optionValueExist = true;
						}
						break;
					case IOption.STRING_LIST :
					case IOption.INCLUDE_PATH :
					case IOption.PREPROCESSOR_SYMBOLS :
					case IOption.LIBRARIES :
						if (opt.getCommand() != null
								&& optionsArr[j].startsWith(opt.getCommand())) {
							optionsList.add(optionsArr[j]);
							optionValueExist = true;
						}
						break;
					case IOption.OBJECTS :
						String userObjStr = getPreferenceStore().getString(
								opt.getId());
						String[] userObjs = BuildToolsSettingsStore
								.parseString(userObjStr);
						for (int m = 0; m < userObjs.length; m++) {
							if (userObjs[m].equalsIgnoreCase(optionsArr[j]))
								optionValueExist = true;
						}
						break;
					default :
						break;
				}
			}
			// If the parsed string does not match with any previous option
			// values then consider this option as a additional build option
			if (!optionValueExist) {
				addnOptions.append(optionsArr[j] + ITool.WHITE_SPACE);
			}
		}
		// Now update the preference store with parsed options
		// Get the options for this tool
		IOption[] options = tool.getOptions();
		for (int k = 0; k < options.length; ++k) {
			IOption opt = options[k];
			String name = opt.getId();
			String listStr = "";
			String[] listVal = null;
			switch (opt.getValueType()) {
				case IOption.BOOLEAN :
					ArrayList optsList = new ArrayList(Arrays
							.asList(optionsArr));
					if (opt.getCommand() != null
							&& !optsList.contains(opt.getCommand()))
						getPreferenceStore().setValue(opt.getId(), false);
					break;
				case IOption.STRING :
					// put the additional options in the compiler flag or
					// linker flag field
					if (opt.getName().equals(COMPILER_FLAGS)
							|| opt.getName().equals(LINKER_FLAGS)) {
						String newOptions = getPreferenceStore().getString(
								opt.getId());
						if (addnOptions.length() > 0) {
							newOptions = newOptions + ITool.WHITE_SPACE
									+ addnOptions.toString().trim();
						}
						getPreferenceStore().setValue(opt.getId(), newOptions);
					}
					break;
				case IOption.STRING_LIST :
				case IOption.INCLUDE_PATH :
				case IOption.PREPROCESSOR_SYMBOLS :
				case IOption.LIBRARIES :
					ArrayList newList = new ArrayList();
					for (int i = 0; i < optionsList.size(); i++) {
						if (opt.getCommand() != null
								&& ((String) optionsList.get(i)).startsWith(opt
										.getCommand())) {
							newList.add(((String) optionsList.get(i))
									.substring(opt.getCommand().length()));
						}
					}
					String[] strlist = new String[newList.size()];
					newList.toArray(strlist);
					newList.clear();
					getPreferenceStore().setValue(opt.getId(),
							BuildToolsSettingsStore.createList(strlist));
					break;
				default :
					break;
			}
		}
	}
	/**
	 * Answers <code>true</code> if the receiver manages settings for the
	 * argument
	 * 
	 * @param tool
	 * @return
	 */
	public boolean isForTool(ITool tool) {
		if (tool != null) {
			return tool.equals(this.tool);
		}
		return false;
	}
	/**
	 * saves all field editors
	 */
	public void storeSettings() {
		super.performOk();
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#performOk()
	 */
	public boolean performOk() {
		// Do the super-class thang
		boolean result = super.performOk();
		//parse and store all build options in the corresponding preference store
		parseAllOptions();
		// Write the preference store values back to the build model
		ITool[] tools = configuration.getTools();
		for (int i = 0; i < tools.length; ++i) {
			if (tools[i] instanceof ToolReference) {
				if (((ToolReference) tools[i]).references(tool)) {
					tool = tools[i];
					break;
				}
			} else if (tools[i].equals(tool))
				break;
		}
		IOption[] options = tool.getOptions();
		for (int i = 0; i < options.length; i++) {
			IOption option = options[i];
			// Transfer value from preference store to options
			switch (option.getValueType()) {
				case IOption.BOOLEAN :
					boolean boolVal = getPreferenceStore().getBoolean(
							option.getId());
					ManagedBuildManager.setOption(configuration, option,
							boolVal);
					break;
				case IOption.ENUMERATED :
					String enumVal = getPreferenceStore().getString(
							option.getId());
					ManagedBuildManager.setOption(configuration, option,
							enumVal);
					break;
				case IOption.STRING :
					String strVal = getPreferenceStore().getString(
							option.getId());
					ManagedBuildManager
							.setOption(configuration, option, strVal);
					break;
				case IOption.STRING_LIST :
				case IOption.INCLUDE_PATH :
				case IOption.PREPROCESSOR_SYMBOLS :
				case IOption.LIBRARIES :
				case IOption.OBJECTS :
					String listStr = getPreferenceStore().getString(
							option.getId());
					String[] listVal = BuildToolsSettingsStore
							.parseString(listStr);
					ManagedBuildManager.setOption(configuration, option,
							listVal);
					break;
				default :
					break;
			}
		}
		// Get the actual value out of the field editor
		String command = getPreferenceStore().getString(tool.getId());
		if (command.length() == 0) {
			return result;
		}
		// The tool may be a reference.
		if (tool instanceof ToolReference) {
			// If so, just set the command in the reference
			((ToolReference) tool).setToolCommand(command);
		} else {
			configuration.setToolCommand(tool, command);
		}
		return result;
	}
}
